@@ -1,4 +1,3 @@
-
 #![allow(dead_code)]
 #![allow(unused_imports)]
 
@@ -88,7 +87,7 @@ fn main() {
 
 fn stat_ticker(verbosity: usize, thread_stopper: Arc<AtomicBool>, status: &mut Arc<IoSlicerStatus>,
                send: &crossbeam_channel::Sender<Option<FileSlice>>,
-               filesend: &crossbeam_channel::Sender<Option<PathBuf>> ) {
+               filesend: &crossbeam_channel::Sender<Option<PathBuf>>) {
     let start_f = Instant::now();
     let startcpu = ProcessTime::now();
 
@@ -156,6 +155,7 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
     let mut total_rowcount = 0usize;
     let mut total_fieldcount = 0usize;
     let mut total_fieldskipped = 0usize;
+    let mut total_lines_skipped = 0usize;
     let mut total_blocks = 0usize;
     let mut total_bytes = 0usize;
 
@@ -353,10 +353,11 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
     // merge the data from the workers
     let mut all_the_maps = vec![];
     for h in worker_handlers {
-        let (map, linecount, fieldcount, fieldskipped) = h.join().unwrap();
+        let (map, linecount, fieldcount, fieldskipped, lines_skipped) = h.join().unwrap();
         total_rowcount += linecount;
         total_fieldcount += fieldcount;
         total_fieldskipped += fieldskipped;
+        total_lines_skipped += lines_skipped;
         all_the_maps.push(map);
     }
     let main_map = if !cfg.no_output {
@@ -390,7 +391,7 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
     let mut thekeys: Vec<_> = main_map.keys().clone().collect();
 
     // may add other comparisons but for now - if ON just this one
-    let chosen_cmp = {
+    let excel_sort_cmp = {
         // partially taken from:
         // https://github.com/DanielKeep/rust-scan-rules/blob/master/src/input.rs#L610-L620
         // but enhanced as a generic Ordering
@@ -433,9 +434,32 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
             Equal
         }
     };
+
+    let count_cmp_asc = {
+        |l_k: &&str, r_k: &&str| -> Ordering {
+            let ks_l = main_map.get(*l_k).unwrap();
+            let ks_r = main_map.get(*r_k).unwrap();
+            ks_l.count.cmp(&ks_r.count)
+        }
+    };
+
+    let count_cmp_dsc = {
+        |l_k: &&str, r_k: &&str| -> Ordering {
+            let ks_l = main_map.get(*l_k).unwrap();
+            let ks_r = main_map.get(*r_k).unwrap();
+            ks_r.count.cmp(&ks_l.count)
+        }
+    };
+
     if !cfg.disable_key_sort {
         let cpu_keysort_s = ProcessTime::now();
-        thekeys.sort_unstable_by(|l, r| { chosen_cmp(&l.as_str(), &r.as_str()) });
+        if cfg.count_dsc {
+            thekeys.sort_unstable_by(|l, r| { count_cmp_dsc(&l.as_str(), &r.as_str()) });
+        } else if cfg.count_asc {
+            thekeys.sort_unstable_by(|l, r| { count_cmp_asc(&l.as_str(), &r.as_str()) });
+        } else {
+            thekeys.sort_unstable_by(|l, r| { excel_sort_cmp(&l.as_str(), &r.as_str()) });
+        }
         if cfg.verbose > 0 {
             let elapsedcpu: Duration = cpu_keysort_s.elapsed();
             let seccpu: f64 = (elapsedcpu.as_secs() as f64) + (elapsedcpu.subsec_nanos() as f64 / 1_000_000_000.0);
@@ -466,7 +490,10 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
     };
 
     if !cfg.no_output {
+        let mut line_count = 0u64;
+        let total_lines_expected = thekeys.len() as u64;
         if !cfg.csv_output {
+            let mut print_skipped = false; // for the stuff after head and before tail
             let mut celltable = Table::new();
             celltable.set_format(*format::consts::FORMAT_NO_BORDER_LINE_SEPARATOR);
             {
@@ -506,7 +533,37 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                 celltable.set_titles(row);
             }
 
-            for ff in thekeys.iter() {
+            // let check_ht: &Fn(u64) -> bool = match (cfg.head.clone(), cfg.tail.clone()) {
+            //     (None, None) =>                                 &|l| -> bool { if l > 0 {false } else { true}},
+            //     (Some(head), None) =>                 &|l| -> bool  { if head < l { false } else { true } },
+            //     (None, Some( tail)) =>                 &|l| -> bool  { if tail < (total_lines_expected-l) { false } else { true }},
+            //     (Some( head), Some( tail)) => &|l| -> bool { { true }},
+            // };
+
+
+            'KEYS_CELL_LOOP: for ff in thekeys.iter() {
+                let cc = main_map.get(*ff).unwrap();
+
+                if let Some(count_ge) = cfg.count_ge {
+                    if cc.count > count_ge { continue 'KEYS_CELL_LOOP; } // to skip
+                }
+                if let Some(count_le) = cfg.count_le {
+                    if cc.count < count_le { continue 'KEYS_CELL_LOOP; } // to skip
+                }
+
+                line_count += 1;
+
+                let print_this_line: bool = match (cfg.head, cfg.tail) {
+                    (None, None) => { true }
+                    (Some(head), None) =>
+                        if head >= line_count { true } else { false },
+                    (None, Some(tail)) => if tail > (total_lines_expected - line_count) { true } else { false }
+                    (Some(head), Some(tail)) => {
+                        if head >= line_count || tail > (total_lines_expected - line_count) { true } else { false }
+                    }
+                };
+                if !print_this_line { continue 'KEYS_CELL_LOOP; }
+
                 let mut vcell = vec![];
                 ff.split(KEY_DEL).for_each(|x| {
                     if x.len() == 0 {
@@ -515,14 +572,14 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                         vcell.push(Cell::new(&x));
                     }
                 });
-                let cc = main_map.get(*ff).unwrap();
+
 
                 if !cfg.no_record_count {
                     vcell.push(Cell::new(&format!("{}", cc.count)));
                 }
                 for x in &cc.nums {
                     match x {
-                        Some(x) =>vcell.push(Cell::new(&format!("{}", x))),
+                        Some(x) => vcell.push(Cell::new(&format!("{}", x))),
                         None => vcell.push(Cell::new(&format!("{}", &cfg.null))),
                     };
                 }
@@ -535,7 +592,7 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                 }
                 for x in &cc.strs {
                     match x {
-                        Some(x) =>vcell.push(Cell::new(&format!("{}", x))),
+                        Some(x) => vcell.push(Cell::new(&format!("{}", x))),
                         None => vcell.push(Cell::new(&format!("{}", &cfg.null))),
                     };
                 }
@@ -597,7 +654,27 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                 line_out.push('\n');
                 writer.write_all(&line_out.as_bytes())?;
             }
-            for ff in thekeys.iter() {
+            'KEYS_CSV_LOOP: for ff in thekeys.iter() {
+                let cc = main_map.get(*ff).unwrap();
+                if let Some(count_ge) = cfg.count_ge {
+                    if cc.count > count_ge { continue 'KEYS_CSV_LOOP; } // to skip
+                }
+                if let Some(count_le) = cfg.count_le {
+                    if cc.count < count_le { continue 'KEYS_CSV_LOOP; } // to skip
+                }
+
+                line_count += 1;
+                let print_this_line: bool = match (cfg.head, cfg.tail) {
+                    (None, None) => { true }
+                    (Some(head), None) =>
+                        if head >= line_count { true } else { false },
+                    (None, Some(tail)) => if tail > (total_lines_expected - line_count) { true } else { false }
+                    (Some(head), Some(tail)) => {
+                        if head >= line_count || tail > (total_lines_expected - line_count) { true } else { false }
+                    }
+                };
+                if !print_this_line { continue 'KEYS_CSV_LOOP; }
+
                 line_out.clear();
                 let keyv: Vec<&str> = ff.split(KEY_DEL).collect();
                 for i in 0..keyv.len() {
@@ -608,13 +685,12 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                     }
                     if i < keyv.len() - 1 { line_out.push_str(&cfg.od); }
                 }
-                let cc = main_map.get(*ff).unwrap();
                 if !cfg.no_record_count {
                     line_out.push_str(&format!("{}{}", &cfg.od, cc.count));
                 }
                 for x in &cc.nums {
                     match x {
-                        Some(x) =>line_out.push_str(&format!("{}{}", &cfg.od, x)),
+                        Some(x) => line_out.push_str(&format!("{}{}", &cfg.od, x)),
                         None => line_out.push_str(&format!("{}{}", &cfg.od, &cfg.null)),
                     };
                 }
@@ -624,7 +700,7 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
                 for x in &cc.strs {
                     line_out.push_str(&cfg.od);
                     match x {
-                        Some(x) =>line_out.push_str(&format!("{}", x)),
+                        Some(x) => line_out.push_str(&format!("{}", x)),
                         None => line_out.push_str(&cfg.null),
                     };
                 }
@@ -667,15 +743,19 @@ fn csv() -> Result<(), Box<dyn std::error::Error>> {
         );
     }
     if total_fieldskipped > 0 {
-        eprintln!("Some fields were skipped {}.  They could not be parsed a numeric.", total_fieldskipped);
+        eprintln!("Some fields were skipped {}, as they could not be parsed as numeric.", total_fieldskipped);
     }
+    if total_lines_skipped > 0 {
+        eprintln!("Note {} records/lines were filtered", total_lines_skipped);
+    }
+
     Ok(())
 }
 
-fn worker_re(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize) {
+fn worker_re(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize, usize) {
     // return lines / fields
     match _worker_re(&send_blocks, cfg, recv) {
-        Ok((map, lines, fields, fieldskipped)) => (map, lines, fields, fieldskipped),
+        Ok((map, lines, fields, fieldskipped, lines_skipped)) => (map, lines, fields, fieldskipped, lines_skipped),
         Err(e) => {
             let err_msg = format!("Unable to process inner file - likely compressed or not UTF8 text: {}", e);
             panic!(err_msg);
@@ -709,7 +789,7 @@ fn _worker_re(
     send_blocks: &crossbeam_channel::Sender<Vec<u8>>,
     cfg: &CliCfg,
     recv: &crossbeam_channel::Receiver<Option<FileSlice>>,
-) -> Result<(MyMap, usize, usize, usize), Box<dyn std::error::Error>> {
+) -> Result<(MyMap, usize, usize, usize, usize), Box<dyn std::error::Error>> {
     // return lines / fields
 
     let mut map = create_map();
@@ -725,6 +805,7 @@ fn _worker_re(
     let mut buff = String::with_capacity(256); // dyn buffer
     let mut fieldcount = 0;
     let mut fieldskipped = 0;
+    let mut lines_skipped = 0;
     let mut rowcount = 0;
     let mut _skipped = 0;
     let mut cl = re.capture_locations();
@@ -759,13 +840,13 @@ fn _worker_re(
                 &fc.block[0..fc.len]
             };
 
-            'LINE_LOOP: for (inner_count, line) in slice.split(|c| *c==b'\n').enumerate() {
+            'LINE_LOOP: for (inner_count, line) in slice.split(|c| *c == b'\n').enumerate() {
                 let line = match std::str::from_utf8(&line) {
                     Err(e) => {
                         ascii_line_fixup_full(line, &mut err_line_fix);
                         eprintln!("skipping line due with UTF8 decoding error (try ISO-8859 option?): {}\n\tline: {}", e, &err_line_fix);
                         continue 'LINE_LOOP;
-                    },
+                    }
                     Ok(line) => line
                 };
 
@@ -795,13 +876,14 @@ fn _worker_re(
                         Some(ref mut sch) => {
                             sch.schema_rec(&v, v.len());
                             if sch.done() {
-                                break 'BLOCK_LOOP
+                                break 'BLOCK_LOOP;
                             }
-                        },
+                        }
                         _ => {
-                            let (fc, fs) = store_rec(&mut buff, "", &v, v.len(), &mut map, &cfg, &mut rowcount);
+                            let (fc, fs, ls) = store_rec(&mut buff, "", &v, v.len(), &mut map, &cfg, &mut rowcount);
                             fieldcount += fc;
                             fieldskipped += fs;
+                            lines_skipped += ls;
                             // cnt_rec += 1;
                         }
                     }
@@ -814,13 +896,13 @@ fn _worker_re(
             send_blocks.send(fc.block)?;
         }
     }
-    if let Some(mut sch) = sch { sch.print_schema(&cfg);}
-    Ok((map, rowcount, fieldcount, fieldskipped))
+    if let Some(mut sch) = sch { sch.print_schema(&cfg); }
+    Ok((map, rowcount, fieldcount, fieldskipped, lines_skipped))
 }
 
-fn worker_csv(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize) {
+fn worker_csv(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize, usize) {
     match _worker_csv(&send_blocks, cfg, recv) {
-        Ok((map, lines, fields, fieldskipped)) => (map, lines, fields, fieldskipped),
+        Ok((map, lines, fields, fieldskipped, lines_skipped)) => (map, lines, fields, fieldskipped, lines_skipped),
         Err(e) => {
             let err_msg = format!("Unable to process inner block - likely compressed or not UTF8 text: {}", e);
             panic!(err_msg);
@@ -832,7 +914,7 @@ fn _worker_csv(
     send_blocks: &crossbeam_channel::Sender<Vec<u8>>,
     cfg: &CliCfg,
     recv: &crossbeam_channel::Receiver<Option<FileSlice>>,
-) -> Result<(MyMap, usize, usize, usize), Box<dyn std::error::Error>> {
+) -> Result<(MyMap, usize, usize, usize, usize), Box<dyn std::error::Error>> {
     // return lines / fields
 
     let mut map = create_map();
@@ -842,6 +924,7 @@ fn _worker_csv(
     let mut buff = String::with_capacity(256); // dyn buffer
     let mut fieldcount = 0;
     let mut fieldskipped = 0;
+    let mut lines_skipped = 0;
     let mut rowcount = 0;
 
     let mut latin_str_buff = String::with_capacity(246);
@@ -887,30 +970,29 @@ fn _worker_csv(
                                 Some(ref mut sch) => {
                                     sch.schema_rec(&v, v.len());
                                     if sch.done() {
-                                        break 'BLOCK_LOOP
+                                        break 'BLOCK_LOOP;
                                     }
-                                },
+                                }
                                 _ => {
-                                    let (fc, fs) = store_rec(&mut buff, "", &record, record.len(), &mut map, &cfg, &mut rowcount);
+                                    let (fc, fs, ls) = store_rec(&mut buff, "", &record, record.len(), &mut map, &cfg, &mut rowcount);
                                     fieldcount += fc;
                                     fieldskipped += fs;
+                                    lines_skipped += ls;
                                 }
                             };
                             v.clear();
-                        },
+                        }
                         Err(e) => {
                             match e.kind() {
-                                csv::ErrorKind::Utf8 {ref err, pos} => {
+                                csv::ErrorKind::Utf8 { ref err, pos } => {
                                     if let Some(pos) = pos {
                                         ascii_line_fixup(&fc.block, pos.byte() as usize, &mut err_line_fix);
-                                        eprintln!("skipping line due to bad UTF8 codes in file (try ISO-8859 option)\n\tline:{}",&err_line_fix);
+                                        eprintln!("skipping line due to bad UTF8 codes in file (try ISO-8859 option)\n\tline:{}", &err_line_fix);
                                     }
-
-                                },
+                                }
                                 _ => eprintln!("skipping line due to err: {:?}", e),
-
                             }
-                        },
+                        }
                     }
                 }
             } else {
@@ -922,31 +1004,29 @@ fn _worker_csv(
                                 Some(ref mut sch) => {
                                     sch.schema_rec(&record, record.len());
                                     if sch.done() {
-                                        break 'BLOCK_LOOP
+                                        break 'BLOCK_LOOP;
                                     }
-                                },
+                                }
                                 _ => {
-                                    let (fc, fs) = store_rec(&mut buff, "", &record, record.len(), &mut map, &cfg, &mut rowcount);
+                                    let (fc, fs, ls) = store_rec(&mut buff, "", &record, record.len(), &mut map, &cfg, &mut rowcount);
                                     fieldcount += fc;
                                     fieldskipped += fs;
+                                    lines_skipped += ls;
                                     cnt_rec += 1;
                                 }
                             };
-                        },
+                        }
                         Err(e) => {
                             match e.kind() {
-                                csv::ErrorKind::Utf8 {ref err, pos} => {
+                                csv::ErrorKind::Utf8 { ref err, pos } => {
                                     if let Some(pos) = pos {
                                         ascii_line_fixup(&fc.block, pos.byte() as usize, &mut err_line_fix);
-                                        eprintln!("skipping line due to bad UTF8 codes in file (try ISO-8859 option)\n\tline:{}",&err_line_fix);
+                                        eprintln!("skipping line due to bad UTF8 codes in file (try ISO-8859 option)\n\tline:{}", &err_line_fix);
                                     }
-
-                                },
+                                }
                                 _ => eprintln!("skipping line due to err: {:?}", e),
-
                             }
-
-                        },
+                        }
                     }
                 }
             }
@@ -955,15 +1035,15 @@ fn _worker_csv(
             send_blocks.send(fc.block)?;
         }
     }
-    if let Some(mut sch) = sch { sch.print_schema(&cfg);}
+    if let Some(mut sch) = sch { sch.print_schema(&cfg); }
 
-    Ok((map, rowcount, fieldcount, fieldskipped))
+    Ok((map, rowcount, fieldcount, fieldskipped, lines_skipped))
 }
 
-fn worker_multi_re(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize) {
+fn worker_multi_re(send_blocks: &crossbeam_channel::Sender<Vec<u8>>, cfg: &CliCfg, recv: &crossbeam_channel::Receiver<Option<FileSlice>>) -> (MyMap, usize, usize, usize, usize) {
     // return lines / fields
     match _worker_multi_re(&send_blocks, cfg, recv) {
-        Ok((map, lines, fields, fieldskipped)) => (map, lines, fields, fieldskipped),
+        Ok((map, lines, fields, fieldskipped, lines_skipped)) => (map, lines, fields, fieldskipped, lines_skipped),
         Err(e) => {
             let err_msg = format!("Unable to process inner file - likely compressed or not UTF8 text: {}", e);
             panic!(err_msg);
@@ -995,7 +1075,7 @@ fn _worker_multi_re(
     send_blocks: &crossbeam_channel::Sender<Vec<u8>>,
     cfg: &CliCfg,
     recv: &crossbeam_channel::Receiver<Option<FileSlice>>,
-) -> Result<(MyMap, usize, usize, usize), Box<dyn std::error::Error>> {
+) -> Result<(MyMap, usize, usize, usize, usize), Box<dyn std::error::Error>> {
     let mut map = create_map();
 
     let mut re_es = vec![];
@@ -1012,6 +1092,7 @@ fn _worker_multi_re(
     let mut buff = String::with_capacity(256); // dyn buffer
     let mut fieldcount = 0;
     let mut fieldskipped = 0;
+    let mut lines_skipped = 0;
     let mut rowcount = 0;
     let mut linecount = 0;
     let mut _skipped = 0;
@@ -1093,13 +1174,14 @@ fn _worker_multi_re(
                             Some(ref mut sch) => {
                                 sch.schema_rec(&v, v.len());
                                 if sch.done() {
-                                    break 'BLOCK_LOOP
+                                    break 'BLOCK_LOOP;
                                 }
-                            },
+                            }
                             _ => {
-                                let (fc, fs) = store_rec(&mut buff, "", &v, v.len(), &mut map, &cfg, &mut rowcount);
+                                let (fc, fs, ls) = store_rec(&mut buff, "", &v, v.len(), &mut map, &cfg, &mut rowcount);
                                 fieldcount += fc;
                                 fieldskipped += fs;
+                                lines_skipped += ls;
                             }
                         }
                         if cfg.verbose > 2 {
@@ -1122,8 +1204,8 @@ fn _worker_multi_re(
             send_blocks.send(fc.block)?;
         }
     }
-    if let Some(mut sch) = sch { sch.print_schema(&cfg);}
-    Ok((map, rowcount, fieldcount, fieldskipped))
+    if let Some(mut sch) = sch { sch.print_schema(&cfg); }
+    Ok((map, rowcount, fieldcount, fieldskipped, lines_skipped))
 }
 
 fn ascii_line_fixup(slice: &[u8], start: usize, str: &mut String) {
